/*************************************************************************
*  Program Name  : FLASH (FLexit Application SHell)                      *
*========================================================================*
*                                                                        *
*  Filename      : conv.c                                                *
*                                                                        *
*  Function      : Blok, signaalgroep en detector nummer conversie.      *
*                                                                        *
*  Date Created  : January 20, 1996                                      *
*                                                                        *
*  Version       : 1.0                                                   *
*                                                                        *
*  Author(s)     : E.E. de Vries (Ed)                                    *
*                                                                        *
*  Files used    :                                                       *
*                                                                        *
*  Date Modified : See RCS identification                                *
*                                                                        *
*  Modifications :                                                       *
*                                                                        *
*  Comments      :                                                       *
*                                                                        *
*------------------------------------------------------------------------*
*  COPYRIGHT (c) 1996 by A.N.D. Identification B.V. Rotterdam.           *
*************************************************************************/

/* -------------------------------------------------------------------- */
/*  INCLUDED FILES                                                      */
/* -------------------------------------------------------------------- */

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include "rws_dim.h" 
#include "evtdisp.h"
#include "crfpbrt.h"
#include "crapdef.h"


/* -------------------------------------------------------------------- */
/*  LOCAL VARIABLES                                                     */
/* -------------------------------------------------------------------- */
#define  MAXDET_SGN          2000   /* max nummer voor sg en detectoren */

/* Conversion tables */
/* ----------------- */
static int convBKItoBKN[MAX_BKI];               /* Blok index to nummer */
static int convSGItoSGN[MAX_SGI];       /* Signaalgroep index to nummer */
static int convDTItoSGN[MAX_DTI];   /* Detector index to signaalgroepnr */
static int convDTItoDTN[MAX_DTI];      /* Detector index to detector nr */

static int convBKNtoBKI[MAX_BKN];               /* Blok nummer to index */
static int convSGNtoSGI[MAXDET_SGN];    /* Signaalgroep nummer to index */

typedef struct 
{
  int    Sg;
  int    Dt;
  int    Value;
}SignaalDetectorValue ;

static SignaalDetectorValue SgDetectorValue[MAXDET_SGN];
typedef int (*fptr)(const void*, const void*);
int numeric (const SignaalDetectorValue *q1, const SignaalDetectorValue *q2);

/* -------------------------------------------------------------------- */
/*  STATIC FUNCTION PROTOTYPES                                          */
/* -------------------------------------------------------------------- */

static void convInitTables(void);
static void convFillTables(void);
static void convCalcTables(void);
static int  convCheckTables(void);


/* -------------------------------------------------------------------- */
/*  IMPLEMENTATION                                                      */
/* -------------------------------------------------------------------- */

/*----------------------------------------------------------------------*/
/* name     : convInit                                                  */
/* function : Initialize, fill and check the conversion tables.         */
/*                                                                      */
/* input    :                                                           */
/* output   :                                                           */
/* return   : 0 (success) or -1 (failure)                               */
/* remarks  : The CRAPDEF.H file will be used to fill the tables.       */
/*                                                                      */
/*----------------------------------------------------------------------*/

int convInit(void)
{
   int retval;

   convInitTables();
   convFillTables();
   retval = convCheckTables();
   convCalcTables();
   return (retval);
}                                                           /* convInit */


/*----------------------------------------------------------------------*/
/* name     : convExit                                                  */
/* function : Exit the conversion tables.                               */
/*                                                                      */
/* input    :                                                           */
/* output   :                                                           */
/* return   :                                                           */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

void convExit(void)
{
}                                                           /* convExit */


/*----------------------------------------------------------------------*/
/* name     : convBkiToBkn                                              */
/* function : Convert block index to block number.                      */
/*                                                                      */
/* input    : index = Block index                                       */
/* output   :                                                           */
/* return   : Block number or -1 (not a valid/defined) index.           */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

int convBkiToBkn(int Index)
{
   if (Index > MAX_BKI)
      return (-1);
   return(convBKItoBKN[Index]);
}                                                       /* convBkiToBkn */


/*----------------------------------------------------------------------*/
/* name     : convSgiToSgn                                              */
/* function : Convert signaalgroep index to signaalgroep number.        */
/*                                                                      */
/* input    : index = Signaalgroep index                                */
/* output   :                                                           */
/* return   : Signaalgroep number or -1 (not a valid/defined) index.    */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

int convSgiToSgn(int Index)
{
   if (Index > MAX_SGI)
      return (-1);
   return(convSGItoSGN[Index]);
}                                                       /* convSgiToSgn */


/*----------------------------------------------------------------------*/
/* name     : convDtiToSgn                                              */
/* function : Convert detector index to signaalgroep number.            */
/*                                                                      */
/* input    : index = Detector index                                    */
/* output   :                                                           */
/* return   : Signaalgroep number or -1 (not a valid/defined) index.    */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

int convDtiToSgn(int Index)
{
   if (Index > MAX_DTI)
      return (-1);
   return(convDTItoSGN[Index]);
}                                                       /* convDtiToSgn */


/*----------------------------------------------------------------------*/
/* name     : convDtiToDtn                                              */
/* function : Convert detector index to detector number.                */
/*                                                                      */
/* input    : index = Detector index                                    */
/* output   :                                                           */
/* return   : Detector number or -1 (not a valid/defined) index.        */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

int convDtiToDtn(int Index)
{
   if (Index > MAX_DTI)
      return (-1);
   return(convDTItoDTN[Index]);
}                                                       /* convDtiToDtn */


/*----------------------------------------------------------------------*/
/* name     : convBknToBki                                              */
/* function : Convert block number to block index.                      */
/*                                                                      */
/* input    : number = Block number                                     */
/* output   :                                                           */
/* return   : Block index or -1 (not a valid/defined) index.            */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

int convBknToBki(int Number)
{
   if (Number > MAX_BKN)
      return (-1);
   return(convBKNtoBKI[Number]);
}                                                       /* convBknToBki */


/*----------------------------------------------------------------------*/
/* name     : convSgnToSgi                                              */
/* function : Convert signaalgroep number to signaalgroep index.        */
/*                                                                      */
/* input    : number = Block number                                     */
/* output   :                                                           */
/* return   : Signaalgroep index or -1 (not a valid/defined) number.    */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

int convSgnToSgi(int Number)
{
   if (Number >= MAX_SGN)
      return (-1);
   return(convSGNtoSGI[Number]);
}                                                       /* convSgnToSgi */


/*----------------------------------------------------------------------*/
/* name     : convDtnToDti                                              */
/* function : Convert signaalgroep+detector number to detector index.   */
/*                                                                      */
/* input    : sgn = Signaalgroep number                                 */
/*            dtn = Detector number                                     */
/* output   :                                                           */
/* return   : Detector index or -1 (not a valid/defined) number.        */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

int convDtnToDti(int Sgn, int Dtn)
{
   SignaalDetectorValue key,*itemptr;
   key.Sg=Sgn; key.Dt=Dtn;
   itemptr = (SignaalDetectorValue *) bsearch (&key, SgDetectorValue, MAXDET_SGN,
                                     sizeof(SgDetectorValue[0]),(fptr)numeric );
   if(itemptr)
   {
     return itemptr->Value;
   }
   else
     return -1;
}                                                       /* convDtnToDti */



/* -------------------------------------------------------------------- */
/*  STATIC FUNCTIONS                                                    */
/* -------------------------------------------------------------------- */

/*----------------------------------------------------------------------*/
/* name     : convInitTables                                            */
/* function : Initialise the conversion tables.                         */
/*                                                                      */
/* input    :                                                           */
/* output   :                                                           */
/* return   :                                                           */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

static void convInitTables(void)
{
   int i;

   for (i=0; i<MAX_BKI; i++)                 /* Reset alle blok nummers */
      convBKItoBKN[i] = -1;
   for (i=0; i<MAX_SGI; i++)         /* Reset alle signaalgroep nummers */
      convSGItoSGN[i] = -1;
   for (i=0; i<MAX_DTI; i++)             /* Reset alle detector nummers */
   {
      convDTItoSGN[i] = -1;
      convDTItoDTN[i] = -1;
   }

   for (i=0; i<MAX_BKN; i++)                 /* Reset alle blok nummers */
      convBKNtoBKI[i] = -1;
   for (i=0; i<MAX_SGN; i++)         /* Reset alle signaalgroep nummers */
      convSGNtoSGI[i] = -1;

   for (i=0; i<MAXDET_SGN; i++)             /* Reset alle signaal indexes en detector indexes */
	 {
     SgDetectorValue[i].Sg    = -1;
     SgDetectorValue[i].Dt    = -1;
     SgDetectorValue[i].Value = -1;
   }
}                                                     /* convInitTables */


/*----------------------------------------------------------------------*/
/* name     : convCalcTables                                            */
/* function : Calculate the remaining conversion tables.                */
/*                                                                      */
/* input    :                                                           */
/* output   :                                                           */
/* return   : 0 (success) or -1 (failure)                               */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

static void convCalcTables(void)
{
   int i;
   int b, s, d;

   for (i=0; i<MAX_BKI; i++)               /* Bereken alle blok nummers */
   {
      if ((b=convBKItoBKN[i]) != -1)
      {
         assert ((b>=0) && (b<MAX_BKN));

         convBKNtoBKI[b] = i;
      }
   }
   for (i=0; i<MAX_SGI; i++)         /* Reset alle signaalgroep nummers */
   {
      if ((s=convSGItoSGN[i]) != -1)
      {
         assert ((s>=0) && (s<MAX_SGN));

         convSGNtoSGI[s] = i;
      }
   }


   for(i=0;i<MAX_DTI;i++)
	 {
      if ((s=convDTItoSGN[i]) != -1)
			{
        assert ((s>=0) && (s<MAXDET_SGN));
        d = convDTItoDTN[i];
        assert ((d>=0) && (d<MAXDET_SGN));
        SgDetectorValue[i].Dt=d;
        SgDetectorValue[i].Sg=s;
        SgDetectorValue[i].Value=i;
      }
	 }
   qsort(SgDetectorValue,MAXDET_SGN ,sizeof(SgDetectorValue[0]), (fptr)numeric);

}                                                     /* convCalcTables */
/*----------------------------------------------------------------------*/
int numeric (const SignaalDetectorValue *q1, const SignaalDetectorValue *q2)
{
   if(q1->Sg == q2->Sg)
     return q1->Dt - q2->Dt;
   return q1->Sg - q2->Sg;
}


/*----------------------------------------------------------------------*/
/* name     : convCheckTables                                           */
/* function : Check the conversion tables.                              */
/*                                                                      */
/* input    :                                                           */
/* output   :                                                           */
/* return   : 0 (success) or -1 (failure)                               */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

static int convCheckTables(void)
{
   int i, j;
   char text[256];

   /* Check if NUMBK is correct */
   /* ------------------------- */
   for (i=0; (i<MAX_BKI) && (convBKItoBKN[i]!=-1); i++);
   if (i != NUMBK)
   {
      sprintf(text, "Het aantal gedefinieerde blokken in de CRAPDEF.H file van de regelaar is %d terwijl NUMBK is %d.", i, NUMBK);
      msgDisplay(EC_MSG_WARNING, text);
   }

   /* Check if NUMSG is correct */
   /* ------------------------- */
   for (i=0; (i<MAX_SGI) && (convSGItoSGN[i]!=-1); i++);
   if (i != NUMSG)
   {
      char text[256];

      sprintf(text, "Het aantal gedefinieerde signaalgroepen in de CRAPDEF.H file van de regelaar is %d terwijl NUMSG is %d.", i, NUMSG);
      msgDisplay(EC_MSG_WARNING, text);
   }


   /* Check if NUMD is correct */
   /* ------------------------- */
   if (NUMD > MAX_DTI)
   {
      sprintf(text, "De define NUMD in de CRAPDEF.H file van de regelaar is %d terwijl het maximum is %d.", NUMD, MAX_DTI);
      msgDisplay(EC_MSG_FATAL, text);
   }

   for (i=0; (i<MAX_DTI) && (convDTItoSGN[i]!=-1); i++);
   if (i > NUMD)
   {
      sprintf(text, "Het aantal gedefinieerde detectoren in de CRAPDEF.H file van de regelaar is %d terwijl NUMD is %d.", i, NUMD);
      msgDisplay(EC_MSG_WARNING, text);
   }
   
   for (j=0; (j<MAX_DTI) && (convDTItoDTN[j]!=-1); j++);
   if (j > NUMD)
   {
      sprintf(text, "Het aantal gedefinieerde detectoren in de CRAPDEF.H file van de regelaar is %d terwijl NUMD is %d.", j, NUMD);
      msgDisplay(EC_MSG_WARNING, text);
   }

   if (i != j)
   {
      sprintf(text, "Interne conversie tabel fout. Het aantal detector-signaalgroep conversies is %d terwijl het aantal detector-detector conversies %d is.", i, j);
      msgDisplay(EC_MSG_WARNING, text);
   }

   return (0);
}                                                    /* convCheckTables */


/*----------------------------------------------------------------------*/
/* name     : convFillTables                                            */
/* function : Fill the conversion tables.                               */
/*                                                                      */
/* input    :                                                           */
/* output   :                                                           */
/* return   :                                                           */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

static void convFillTables(void)
{
   /* Vul blok conversie tabel */
   /* ------------------------ */

#ifdef BK0
   convBKItoBKN[BK0] = 0;
#endif
#ifdef BK1
   convBKItoBKN[BK1] = 1;
#endif
#ifdef BK2
   convBKItoBKN[BK2] = 2;
#endif
#ifdef BK3
   convBKItoBKN[BK3] = 3;
#endif
#ifdef BK4
   convBKItoBKN[BK4] = 4;
#endif
#ifdef BK5
   convBKItoBKN[BK5] = 5;
#endif
#ifdef BK6
   convBKItoBKN[BK6] = 6;
#endif
#ifdef BK7
   convBKItoBKN[BK7] = 7;
#endif
#ifdef BK8
   convBKItoBKN[BK8] = 8;
#endif
#ifdef BK9
   convBKItoBKN[BK9] = 9;
#endif
